import lepcpy as le
import wx
import uuid
from enum import Enum


# Emulation key kind
class EmulationKeyKind(Enum):
    KEY = 0
    TOGGLE = 1
    SPACER = 2


# Emulation model kind
class EmulationModelKind(Enum):
    LE8200 = 'LE-8200, LE-8200A'
    LEX500XR = 'LE-3500XR, LR-2500XR'
    LE8586 = 'LE-8500XR-RT, LE-8500X-RT, LE-8600XR, LE-8600X, LE-8500XR, LE-8500X'


# Emulation keys
EMULATION_KEYS = {
    EmulationModelKind.LE8200: [
        [  # Line 1
            (le.LE_REMOTE_KEY_F1, 'F1', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_F2, 'F2', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_F3, 'F3', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_F4, 'F4', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_F5, 'F5', EmulationKeyKind.KEY),
        ],
        [  # Line 2
            (le.LE_REMOTE_KEY_C, 'C .!?', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_D, 'B ,;:', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_E, 'E (<[{', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_F, 'F )>]}', EmulationKeyKind.KEY),
            (None, '', EmulationKeyKind.SPACER),
            (le.LE_REMOTE_KEY_MENU, 'Menu', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_DISPLAY, 'Data', EmulationKeyKind.KEY),
            (None, '', EmulationKeyKind.SPACER),
            (le.LE_REMOTE_KEY_STOP, 'Stop', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_RUN, 'Run', EmulationKeyKind.KEY),
        ],
        [  # Line 3
            (le.LE_REMOTE_KEY_8, '8 "\'^~', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_9, '9 $￥@', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_A, 'A -+=', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_B, 'B /*\\', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_DELETE, 'Top\nDel', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_LOADSAVE, 'File', EmulationKeyKind.KEY),
            (None, '', EmulationKeyKind.SPACER),
            (le.LE_REMOTE_KEY_UP, '↑', EmulationKeyKind.KEY),
            (None, '', EmulationKeyKind.SPACER),
            (le.LE_REMOTE_KEY_PAGEUP, 'Page\nUp', EmulationKeyKind.KEY),
        ],
        [  # Line 4
            (le.LE_REMOTE_KEY_4, '4 STU', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_5, '5 VWX', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_6, '6 YZΔ', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_7, '7 #%&|', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_BOTTOM, 'End\nX', EmulationKeyKind.KEY),
            (None, '', EmulationKeyKind.SPACER),
            (le.LE_REMOTE_KEY_LEFT, '←', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_ENTER, 'Enter', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_RIGHT, '→', EmulationKeyKind.KEY),
            (None, '', EmulationKeyKind.SPACER),
        ],
        [  # Line 5
            (le.LE_REMOTE_KEY_0, '0 GHI', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_1, '1 JKL', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_2, '2 MNO', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_3, '3 PQR', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_SHIFTON, 'Shift', EmulationKeyKind.TOGGLE),
            (le.LE_REMOTE_KEY_PRINT, 'Print', EmulationKeyKind.KEY),
            (None, '', EmulationKeyKind.SPACER),
            (le.LE_REMOTE_KEY_DOWN, '↓', EmulationKeyKind.KEY),
            (None, '', EmulationKeyKind.SPACER),
            (le.LE_REMOTE_KEY_PAGEDOWN, 'Page\nDown', EmulationKeyKind.KEY),
        ]
    ],
    EmulationModelKind.LEX500XR: [
        [  # Line 1
            (le.LE_REMOTE_KEY_C, 'C', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_D, 'B', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_E, 'E', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_F, 'F', EmulationKeyKind.KEY),
            (None, '', EmulationKeyKind.SPACER),
            (le.LE_REMOTE_KEY_ESC, 'ESC', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_MENU, 'MENU', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_STOP, 'STOP', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_RUN, 'RUN', EmulationKeyKind.KEY),
        ],
        [  # Line 3
            (le.LE_REMOTE_KEY_8, '8', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_9, '9', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_A, 'A', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_B, 'B', EmulationKeyKind.KEY),
            (None, '', EmulationKeyKind.SPACER),
            (le.LE_REMOTE_KEY_DELETE, 'TOP\nDEL', EmulationKeyKind.KEY),
            (None, '', EmulationKeyKind.SPACER),
            (le.LE_REMOTE_KEY_UP, '↑', EmulationKeyKind.KEY),
            (None, '', EmulationKeyKind.SPACER),
        ],
        [  # Line 4
            (le.LE_REMOTE_KEY_4, '4', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_5, '5', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_6, '6', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_7, '7', EmulationKeyKind.KEY),
            (None, '', EmulationKeyKind.SPACER),
            (le.LE_REMOTE_KEY_BOTTOM, 'END\nX', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_LEFT, '←', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_ENTER, 'ENTER', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_RIGHT, '→', EmulationKeyKind.KEY),
        ],
        [  # Line 5
            (le.LE_REMOTE_KEY_0, '0', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_1, '1', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_2, '2', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_3, '3', EmulationKeyKind.KEY),
            (None, '', EmulationKeyKind.SPACER),
            (le.LE_REMOTE_KEY_SHIFTON, 'SHIFT', EmulationKeyKind.TOGGLE),
            (None, '', EmulationKeyKind.SPACER),
            (le.LE_REMOTE_KEY_DOWN, '↓', EmulationKeyKind.KEY),
            (None, '', EmulationKeyKind.SPACER),
        ],
    ],
    EmulationModelKind.LE8586: [
        [  # Line 1
            (le.LE_REMOTE_KEY_C, 'C', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_D, 'B', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_E, 'E', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_F, 'F', EmulationKeyKind.KEY),
            (None, '', EmulationKeyKind.SPACER),
            (le.LE_REMOTE_KEY_ESC, 'ESC', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_MENU, 'MENU', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_STOP, 'STOP', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_RUN, 'RUN', EmulationKeyKind.KEY),
        ],
        [  # Line 3
            (le.LE_REMOTE_KEY_8, '8', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_9, '9', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_A, 'A', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_B, 'B', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_DELETE, 'TOP\nDEL', EmulationKeyKind.KEY),
            (None, '', EmulationKeyKind.SPACER),
            (None, '', EmulationKeyKind.SPACER),
            (le.LE_REMOTE_KEY_UP, '↑', EmulationKeyKind.KEY),
            (None, '', EmulationKeyKind.SPACER),
        ],
        [  # Line 4
            (le.LE_REMOTE_KEY_4, '4', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_5, '5', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_6, '6', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_7, '7', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_BOTTOM, 'END\nX', EmulationKeyKind.KEY),
            (None, '', EmulationKeyKind.SPACER),
            (le.LE_REMOTE_KEY_LEFT, '←', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_ENTER, 'ENTER', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_RIGHT, '→', EmulationKeyKind.KEY),
        ],
        [  # Line 5
            (le.LE_REMOTE_KEY_0, '0', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_1, '1', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_2, '2', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_3, '3', EmulationKeyKind.KEY),
            (le.LE_REMOTE_KEY_SHIFTON, 'SHIFT', EmulationKeyKind.TOGGLE),
            (None, '', EmulationKeyKind.SPACER),
            (None, '', EmulationKeyKind.SPACER),
            (le.LE_REMOTE_KEY_DOWN, '↓', EmulationKeyKind.KEY),
            (None, '', EmulationKeyKind.SPACER),
        ],
    ]
}


'''
    Serial Dialog
    
    Input a serial number and select LE Analyzer model
'''


class SerialDialog(wx.Dialog):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.txt_serial = wx.TextCtrl(self, wx.ID_ANY, '')
        self.cmb_model = wx.ComboBox(self, wx.ID_ANY, style=wx.CB_READONLY, choices=[
                                     m.value for m in EmulationModelKind])
        self.btn_ok = wx.Button(self, wx.ID_ANY, 'OK')
        self.btn_ok.Bind(wx.EVT_BUTTON, self.onPressed)

        sizer_input = wx.BoxSizer(wx.VERTICAL)
        sizer_input.AddSpacer(10)
        sizer_input.Add(wx.StaticText(self, label='Serial: '))
        sizer_input.Add(self.txt_serial)
        sizer_input.AddSpacer(10)
        sizer_input.Add(wx.StaticText(self, label='Model: '))
        sizer_input.Add(self.cmb_model)
        sizer_input.AddSpacer(10)

        sizer_btn = wx.BoxSizer(wx.VERTICAL)
        sizer_btn.AddSpacer(10)
        sizer_btn.Add(self.btn_ok)
        sizer_btn.AddSpacer(10)

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.AddSpacer(10)
        sizer.Add(sizer_input)
        sizer.AddSpacer(10)
        sizer.Add(sizer_btn, flag=wx.ALIGN_BOTTOM)
        sizer.AddSpacer(10)

        self.SetSizerAndFit(sizer)
        self.Center(wx.BOTH)

    def onPressed(self, event):
        self.Close()

    def getSerial(self):
        return self.txt_serial.GetValue()

    def getModel(self):
        model = self.cmb_model.GetStringSelection()
        if model == '':
            return EmulationModelKind.LE8200
        else:
            return EmulationModelKind(model)


'''
    Bitmap Panel
'''


class BitmapPanel(wx.Panel):
    def __init__(self, parent, bitmap):
        super().__init__(parent, wx.ID_ANY, size=bitmap.GetSize())
        self.bitmap = bitmap
        self.Bind(wx.EVT_PAINT, self.onPaint)

    def onPaint(self, event):
        dc = wx.PaintDC(self)
        self.drawDc(dc)

    def paint(self):
        dc = wx.ClientDC(self)
        self.drawDc(dc)
        self.Refresh()

    def drawDc(self, dc):
        if not self.bitmap.IsOk():
            return
        dc.DrawBitmap(self.bitmap, 0, 0, False)


'''
    Main
'''


class MyApp(wx.Frame):
    TITLE = 'LE Analyzer Emu'

    ID_TIMER_SCREENSHOT = 1000
    ID_TIMER_REFRESH = 1001

    # Take a snapshot -> 10ms delay -> display (1 Hz)
    # For serial connection, please set longer values
    DURATION_SCREENSHOT = 1000
    DELAY_REFRESH = 10
    DELAY_SCREENSHOT_BY_KEY_ACTION = 350

    def __init__(self):
        super().__init__(None, wx.ID_ANY, title=self.TITLE,
                         style=wx.SYSTEM_MENU | wx.CAPTION | wx.CLOSE_BOX)
        self.is_shift = False

        '''
          Get the serial number
        '''
        dialog = SerialDialog(self, title=self.TITLE + ' - Serial')
        serial = ''  # Auto
        model = ''
        try:
            dialog.ShowModal()
            serial = dialog.getSerial()
            model = dialog.getModel()
        finally:
            dialog.Destroy()

        if model == '':
            self.Close()
            return

        '''
          Connect to LE Analyzer
        '''
        if not le.connect(serial):
            # Connect error
            wx.MessageDialog(self, 'Failed to connect LE Analyzer.',
                             self.TITLE + ' - Error', wx.ICON_ERROR).ShowModal()
            self.Close()
            return

        # Take a screenshot to get bitmap size
        self.bitmap_path = str(uuid.uuid1()) + '.bmp'
        le.getScreen(self.bitmap_path)

        '''
          Create window components
        '''
        # Create menubar
        file_menu = wx.Menu()
        file_menu.Append(wx.ID_EXIT, 'E&xit\tAlt-X', 'Quit this program')

        menu_bar = wx.MenuBar()
        menu_bar.Append(file_menu, '&File')
        self.SetMenuBar(menu_bar)

        # Create buttons
        self.panel_btns = wx.Panel(self, wx.ID_ANY)
        sizer_btns = wx.BoxSizer(wx.VERTICAL)
        btn_size = wx.Size(65, 35)
        for line in EMULATION_KEYS[model]:
            sizer = wx.BoxSizer(wx.HORIZONTAL)
            for key, label, kind in line:
                match kind:
                    case EmulationKeyKind.KEY:
                        sizer.Add(wx.Button(self.panel_btns,
                                  100 + key, label, size=btn_size))
                    case EmulationKeyKind.TOGGLE:
                        sizer.Add(wx.ToggleButton(self.panel_btns,
                                  100 + key, label, size=btn_size))
                    case _:
                        sizer.Add(btn_size.GetWidth(), btn_size.GetHeight(), 0)
            sizer_btns.Add(sizer, flag=wx.ALIGN_CENTER)

        self.panel_btns.SetSizer(sizer_btns)
        self.panel_btns.Center(wx.BOTH)

        # Create screenshot panel
        self.bitmap = wx.Bitmap(self.bitmap_path)
        self.panel_cap = BitmapPanel(self, self.bitmap)
        self.panel_cap.paint()

        # Layout
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.panel_cap, flag=wx.ALIGN_CENTER)
        sizer.Add(self.panel_btns, flag=wx.EXPAND)
        self.SetSizerAndFit(sizer)
        self.Centre(wx.BOTH)

        # Create screenshot timer
        self.timer_screenshot = wx.Timer(self, self.ID_TIMER_SCREENSHOT)
        self.timer_refresh = wx.Timer(self, self.ID_TIMER_REFRESH)

        # Add events
        self.Bind(wx.EVT_CLOSE, self.onCloseWindow)
        self.Bind(wx.EVT_MENU, self.onMenuSelect)
        self.Bind(wx.EVT_BUTTON, self.onKeyPressed)
        self.Bind(wx.EVT_TOGGLEBUTTON, self.onShiftPressed)
        self.Bind(wx.EVT_TIMER, self.onTimer)

        # Start screenshot timer
        self.timer_screenshot.StartOnce(self.DURATION_SCREENSHOT)

        # Show this panel
        self.Show()

    def onCloseWindow(self, event):
        le.disconnect()
        self.Destroy()

    def onMenuSelect(self, event):
        if event.GetId() == wx.ID_EXIT:
            self.Close()

    def onShiftPressed(self, event):
        key = event.GetId() - 100
        if key == le.LE_REMOTE_KEY_SHIFTON:
            self.is_shift = event.GetEventObject().GetValue()

    def onKeyPressed(self, event):
        key = event.GetId() - 100
        if self.is_shift:
            key = key | le.LE_REMOTE_KEY_SHIFT
        le.emulationKey(key)
        # Cancel current timer and take a snapshot after a few ms
        self.timer_screenshot.Stop()
        self.timer_screenshot.StartOnce(self.DELAY_SCREENSHOT_BY_KEY_ACTION)

    def onTimer(self, event):
        match event.GetId():
            case self.ID_TIMER_SCREENSHOT:
                le.getScreen(self.bitmap_path)
                self.timer_refresh.StartOnce(self.DELAY_REFRESH)
            case self.ID_TIMER_REFRESH:
                self.bitmap.LoadFile(self.bitmap_path)
                self.panel_cap.paint()
                self.timer_screenshot.StartOnce(self.DURATION_SCREENSHOT)


app = wx.App()
MyApp()
app.MainLoop()
